'use strict';

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _Object$defineProperty = require('babel-runtime/core-js/object/define-property')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;
exports['default'] = graphql;

var _parse = require('./parse');

var _parse2 = _interopRequireDefault(_parse);

var _traverse = require('./traverse');

var _traverse2 = _interopRequireDefault(_traverse);

function join(parts) {
  var result = parts[0] || '';
  for (var i = 1; i < parts.length; i++) {
    result += '&' + (i - 1) + parts[i];
  }

  return result;
}

function graphql(strings) {
  for (var _len = arguments.length, values = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    values[_key - 1] = arguments[_key];
  }

  var source = join(strings);
  var ast = _parse2['default'](source);

  return function (params) {
    return _traverse2['default'](ast, new TaggedTemplateVisitor(params, values));
  };
}

var rootSymbol = '@@root';
var nameSymbol = '@@name';

var TaggedTemplateVisitor = (function () {
  function TaggedTemplateVisitor(params, quasis) {
    _classCallCheck(this, TaggedTemplateVisitor);

    this.params = params;
    this.quasis = quasis;
  }

  TaggedTemplateVisitor.prototype.transformFields = function transformFields(fields) {
    var obj = {};

    for (var i = 0; i < fields.length; i++) {
      var f = fields[i];
      if (f.hasOwnProperty(rootSymbol)) {
        for (var field in f) {
          obj[field] = f[field];
        }
      } else {
        obj[f[nameSymbol]] = f;
      }
    }

    return obj;
  };

  TaggedTemplateVisitor.prototype.Query = function Query(node) {
    var query = this.transformFields(node.fields);
    _Object$defineProperty(query, rootSymbol, { value: true });

    return query;
  };

  TaggedTemplateVisitor.prototype.Field = function Field(node) {
    var field = {};
    _Object$defineProperty(field, nameSymbol, { value: node.name });

    if (node.alias) {
      field.alias = node.alias;
    }

    if (node.params.length > 0) {
      field.params = {};

      for (var i = 0; i < node.params.length; i++) {
        var arg = node.params[i];
        field.params[arg.name] = arg.value;
      }
    }

    if (node.fields.length > 0) {
      field.fields = this.transformFields(node.fields);
    }

    return field;
  };

  TaggedTemplateVisitor.prototype.Literal = function Literal(node) {
    return node.value;
  };

  TaggedTemplateVisitor.prototype.Reference = function Reference(node) {
    return this.quasis[node.name];
  };

  TaggedTemplateVisitor.prototype.Variable = function Variable(node) {
    return this.params[node.name];
  };

  return TaggedTemplateVisitor;
})();

module.exports = exports['default'];