'use strict';

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

exports.__esModule = true;

var _tokenizer = require('./tokenizer');

var _tokenizer2 = _interopRequireDefault(_tokenizer);

var Parser = (function (_Tokenizer) {
  function Parser() {
    _classCallCheck(this, Parser);

    _Tokenizer.apply(this, arguments);
  }

  _inherits(Parser, _Tokenizer);

  Parser.prototype.match = function match(type) {
    return this.lookahead.type === type;
  };

  Parser.prototype.eat = function eat(type) {
    if (this.match(type)) {
      return this.lex();
    }

    return null;
  };

  Parser.prototype.expect = function expect(type) {
    if (this.match(type)) {
      return this.lex();
    }

    throw this.createUnexpected(this.lookahead);
  };

  Parser.prototype.parseQuery = function parseQuery() {
    return { type: 'Query', fields: this.parseFieldList() };
  };

  Parser.prototype.parseIdentifier = function parseIdentifier() {
    return this.expect(_tokenizer.TokenType.IDENTIFIER).value;
  };

  Parser.prototype.parseFieldList = function parseFieldList() {
    this.expect(_tokenizer.TokenType.LBRACE);

    var fields = [];
    var first = true;

    while (!this.match(_tokenizer.TokenType.RBRACE) && !this.end()) {
      if (first) {
        first = false;
      } else {
        this.expect(_tokenizer.TokenType.COMMA);
      }

      if (this.match(_tokenizer.TokenType.AMP)) {
        fields.push(this.parseReference());
      } else {
        fields.push(this.parseField());
      }
    }

    this.expect(_tokenizer.TokenType.RBRACE);
    return fields;
  };

  Parser.prototype.parseField = function parseField() {
    var name = this.parseIdentifier();
    var params = this.match(_tokenizer.TokenType.LPAREN) ? this.parseArgumentList() : [];
    var alias = this.eat(_tokenizer.TokenType.AS) ? this.parseIdentifier() : null;
    var fields = this.match(_tokenizer.TokenType.LBRACE) ? this.parseFieldList() : [];

    return { type: 'Field', name: name, alias: alias, params: params, fields: fields };
  };

  Parser.prototype.parseArgumentList = function parseArgumentList() {
    var args = [];
    var first = true;

    this.expect(_tokenizer.TokenType.LPAREN);

    while (!this.match(_tokenizer.TokenType.RPAREN) && !this.end()) {
      if (first) {
        first = false;
      } else {
        this.expect(_tokenizer.TokenType.COMMA);
      }

      args.push(this.parseArgument());
    }

    this.expect(_tokenizer.TokenType.RPAREN);
    return args;
  };

  Parser.prototype.parseArgument = function parseArgument() {
    var name = this.parseIdentifier();
    this.expect(_tokenizer.TokenType.COLON);
    var value = this.parseValue();

    return { type: 'Argument', name: name, value: value };
  };

  Parser.prototype.parseValue = function parseValue() {
    switch (this.lookahead.type) {
      case _tokenizer.TokenType.AMP:
        return this.parseReference();

      case _tokenizer.TokenType.LT:
        return this.parseVariable();

      case _tokenizer.TokenType.NUMBER:
      case _tokenizer.TokenType.STRING:
        return { type: 'Literal', value: this.lex().value };

      case _tokenizer.TokenType.NULL:
      case _tokenizer.TokenType.TRUE:
      case _tokenizer.TokenType.FALSE:
        return { type: 'Literal', value: JSON.parse(this.lex().value) };
    }

    throw this.createUnexpected(this.lookahead);
  };

  Parser.prototype.parseReference = function parseReference() {
    this.expect(_tokenizer.TokenType.AMP);

    if (this.match(_tokenizer.TokenType.NUMBER) || this.match(_tokenizer.TokenType.IDENTIFIER)) {
      return { type: 'Reference', name: this.lex().value };
    }

    throw this.createUnexpected(this.lookahead);
  };

  Parser.prototype.parseVariable = function parseVariable() {
    this.expect(_tokenizer.TokenType.LT);
    var name = this.expect(_tokenizer.TokenType.IDENTIFIER).value;
    this.expect(_tokenizer.TokenType.GT);

    return { type: 'Variable', name: name };
  };

  return Parser;
})(_tokenizer2['default']);

exports['default'] = Parser;
module.exports = exports['default'];